
from .server import ABCIServer
from .application import BaseApplication, CodeTypeOk
from .types_pb2 import (
    ResponseInitChain,
    ResponseInfo,
    ResponseSetOption,
    ResponseDeliverTx,
    ResponseCheckTx,
    ResponseQuery,
    ResponseBeginBlock,
    ResponseEndBlock,
    ResponseCommit,
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: types.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from github.com.gogo.protobuf.gogoproto import gogo_pb2 as github_dot_com_dot_gogo_dot_protobuf_dot_gogoproto_dot_gogo__pb2
from github.com.tendermint.tmlibs.common import types_pb2 as github_dot_com_dot_tendermint_dot_tmlibs_dot_common_dot_types__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='types.proto',
  package='types',
  syntax='proto3',
  serialized_pb=_b('\n\x0btypes.proto\x12\x05types\x1a-github.com/gogo/protobuf/gogoproto/gogo.proto\x1a/github.com/tendermint/tmlibs/common/types.proto\"\xe4\x03\n\x07Request\x12\"\n\x04\x65\x63ho\x18\x02 \x01(\x0b\x32\x12.types.RequestEchoH\x00\x12$\n\x05\x66lush\x18\x03 \x01(\x0b\x32\x13.types.RequestFlushH\x00\x12\"\n\x04info\x18\x04 \x01(\x0b\x32\x12.types.RequestInfoH\x00\x12-\n\nset_option\x18\x05 \x01(\x0b\x32\x17.types.RequestSetOptionH\x00\x12-\n\ninit_chain\x18\x06 \x01(\x0b\x32\x17.types.RequestInitChainH\x00\x12$\n\x05query\x18\x07 \x01(\x0b\x32\x13.types.RequestQueryH\x00\x12/\n\x0b\x62\x65gin_block\x18\x08 \x01(\x0b\x32\x18.types.RequestBeginBlockH\x00\x12)\n\x08\x63heck_tx\x18\t \x01(\x0b\x32\x15.types.RequestCheckTxH\x00\x12-\n\ndeliver_tx\x18\x13 \x01(\x0b\x32\x17.types.RequestDeliverTxH\x00\x12+\n\tend_block\x18\x0b \x01(\x0b\x32\x16.types.RequestEndBlockH\x00\x12&\n\x06\x63ommit\x18\x0c \x01(\x0b\x32\x14.types.RequestCommitH\x00\x42\x07\n\x05value\"\x1e\n\x0bRequestEcho\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x0e\n\x0cRequestFlush\"\x1e\n\x0bRequestInfo\x12\x0f\n\x07version\x18\x01 \x01(\t\".\n\x10RequestSetOption\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"W\n\x10RequestInitChain\x12*\n\nvalidators\x18\x01 \x03(\x0b\x32\x10.types.ValidatorB\x04\xc8\xde\x1f\x00\x12\x17\n\x0f\x61pp_state_bytes\x18\x02 \x01(\x0c\"I\n\x0cRequestQuery\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x0e\n\x06height\x18\x03 \x01(\x03\x12\r\n\x05prove\x18\x04 \x01(\x08\"\x96\x01\n\x11RequestBeginBlock\x12\x0c\n\x04hash\x18\x01 \x01(\x0c\x12#\n\x06header\x18\x02 \x01(\x0b\x32\r.types.HeaderB\x04\xc8\xde\x1f\x00\x12\x19\n\x11\x61\x62sent_validators\x18\x03 \x03(\x05\x12\x33\n\x14\x62yzantine_validators\x18\x04 \x03(\x0b\x32\x0f.types.EvidenceB\x04\xc8\xde\x1f\x00\"\x1c\n\x0eRequestCheckTx\x12\n\n\x02tx\x18\x01 \x01(\x0c\"\x1e\n\x10RequestDeliverTx\x12\n\n\x02tx\x18\x01 \x01(\x0c\"!\n\x0fRequestEndBlock\x12\x0e\n\x06height\x18\x01 \x01(\x03\"\x0f\n\rRequestCommit\"\x9f\x04\n\x08Response\x12-\n\texception\x18\x01 \x01(\x0b\x32\x18.types.ResponseExceptionH\x00\x12#\n\x04\x65\x63ho\x18\x02 \x01(\x0b\x32\x13.types.ResponseEchoH\x00\x12%\n\x05\x66lush\x18\x03 \x01(\x0b\x32\x14.types.ResponseFlushH\x00\x12#\n\x04info\x18\x04 \x01(\x0b\x32\x13.types.ResponseInfoH\x00\x12.\n\nset_option\x18\x05 \x01(\x0b\x32\x18.types.ResponseSetOptionH\x00\x12.\n\ninit_chain\x18\x06 \x01(\x0b\x32\x18.types.ResponseInitChainH\x00\x12%\n\x05query\x18\x07 \x01(\x0b\x32\x14.types.ResponseQueryH\x00\x12\x30\n\x0b\x62\x65gin_block\x18\x08 \x01(\x0b\x32\x19.types.ResponseBeginBlockH\x00\x12*\n\x08\x63heck_tx\x18\t \x01(\x0b\x32\x16.types.ResponseCheckTxH\x00\x12.\n\ndeliver_tx\x18\n \x01(\x0b\x32\x18.types.ResponseDeliverTxH\x00\x12,\n\tend_block\x18\x0b \x01(\x0b\x32\x17.types.ResponseEndBlockH\x00\x12\'\n\x06\x63ommit\x18\x0c \x01(\x0b\x32\x15.types.ResponseCommitH\x00\x42\x07\n\x05value\"\"\n\x11ResponseException\x12\r\n\x05\x65rror\x18\x01 \x01(\t\"\x1f\n\x0cResponseEcho\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x0f\n\rResponseFlush\"e\n\x0cResponseInfo\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x19\n\x11last_block_height\x18\x03 \x01(\x03\x12\x1b\n\x13last_block_app_hash\x18\x04 \x01(\x0c\"<\n\x11ResponseSetOption\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\x0b\n\x03log\x18\x03 \x01(\t\x12\x0c\n\x04info\x18\x04 \x01(\t\"\x13\n\x11ResponseInitChain\"\x82\x01\n\rResponseQuery\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\x0b\n\x03log\x18\x03 \x01(\t\x12\x0c\n\x04info\x18\x04 \x01(\t\x12\r\n\x05index\x18\x05 \x01(\x03\x12\x0b\n\x03key\x18\x06 \x01(\x0c\x12\r\n\x05value\x18\x07 \x01(\x0c\x12\r\n\x05proof\x18\x08 \x01(\x0c\x12\x0e\n\x06height\x18\t \x01(\x03\"\x14\n\x12ResponseBeginBlock\"\xc9\x01\n\x0fResponseCheckTx\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0b\n\x03log\x18\x03 \x01(\t\x12\x0c\n\x04info\x18\x04 \x01(\t\x12\x12\n\ngas_wanted\x18\x05 \x01(\x03\x12\x10\n\x08gas_used\x18\x06 \x01(\x03\x12\x34\n\x04tags\x18\x07 \x03(\x0b\x32\x0e.common.KVPairB\x16\xc8\xde\x1f\x00\xea\xde\x1f\x0etags,omitempty\x12#\n\x03\x66\x65\x65\x18\x08 \x01(\x0b\x32\x10.common.KI64PairB\x04\xc8\xde\x1f\x00\"\xcb\x01\n\x11ResponseDeliverTx\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0b\n\x03log\x18\x03 \x01(\t\x12\x0c\n\x04info\x18\x04 \x01(\t\x12\x12\n\ngas_wanted\x18\x05 \x01(\x03\x12\x10\n\x08gas_used\x18\x06 \x01(\x03\x12\x34\n\x04tags\x18\x07 \x03(\x0b\x32\x0e.common.KVPairB\x16\xc8\xde\x1f\x00\xea\xde\x1f\x0etags,omitempty\x12#\n\x03\x66\x65\x65\x18\x08 \x01(\x0b\x32\x10.common.KI64PairB\x04\xc8\xde\x1f\x00\"~\n\x10ResponseEndBlock\x12\x31\n\x11validator_updates\x18\x01 \x03(\x0b\x32\x10.types.ValidatorB\x04\xc8\xde\x1f\x00\x12\x37\n\x17\x63onsensus_param_updates\x18\x02 \x01(\x0b\x32\x16.types.ConsensusParams\"\x1e\n\x0eResponseCommit\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\x81\x01\n\x0f\x43onsensusParams\x12$\n\nblock_size\x18\x01 \x01(\x0b\x32\x10.types.BlockSize\x12\x1e\n\x07tx_size\x18\x02 \x01(\x0b\x32\r.types.TxSize\x12(\n\x0c\x62lock_gossip\x18\x03 \x01(\x0b\x32\x12.types.BlockGossip\"@\n\tBlockSize\x12\x11\n\tmax_bytes\x18\x01 \x01(\x05\x12\x0f\n\x07max_txs\x18\x02 \x01(\x05\x12\x0f\n\x07max_gas\x18\x03 \x01(\x03\",\n\x06TxSize\x12\x11\n\tmax_bytes\x18\x01 \x01(\x05\x12\x0f\n\x07max_gas\x18\x02 \x01(\x03\",\n\x0b\x42lockGossip\x12\x1d\n\x15\x62lock_part_size_bytes\x18\x01 \x01(\x05\"\xea\x01\n\x06Header\x12\x1d\n\x08\x63hain_id\x18\x01 \x01(\tB\x0b\xe2\xde\x1f\x07\x43hainID\x12\x0e\n\x06height\x18\x02 \x01(\x03\x12\x0c\n\x04time\x18\x03 \x01(\x03\x12\x0f\n\x07num_txs\x18\x04 \x01(\x05\x12:\n\rlast_block_id\x18\x05 \x01(\x0b\x32\x0e.types.BlockIDB\x13\xe2\xde\x1f\x0bLastBlockID\xc8\xde\x1f\x00\x12\x18\n\x10last_commit_hash\x18\x06 \x01(\x0c\x12\x11\n\tdata_hash\x18\x07 \x01(\x0c\x12\x17\n\x0fvalidators_hash\x18\x08 \x01(\x0c\x12\x10\n\x08\x61pp_hash\x18\t \x01(\x0c\"B\n\x07\x42lockID\x12\x0c\n\x04hash\x18\x01 \x01(\x0c\x12)\n\x05parts\x18\x02 \x01(\x0b\x32\x14.types.PartSetHeaderB\x04\xc8\xde\x1f\x00\",\n\rPartSetHeader\x12\r\n\x05total\x18\x01 \x01(\x05\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\"+\n\tValidator\x12\x0f\n\x07pub_key\x18\x01 \x01(\x0c\x12\r\n\x05power\x18\x02 \x01(\x03\"+\n\x08\x45vidence\x12\x0f\n\x07pub_key\x18\x01 \x01(\x0c\x12\x0e\n\x06height\x18\x02 \x01(\x03\x32\x8c\x05\n\x0f\x41\x42\x43IApplication\x12/\n\x04\x45\x63ho\x12\x12.types.RequestEcho\x1a\x13.types.ResponseEcho\x12\x32\n\x05\x46lush\x12\x13.types.RequestFlush\x1a\x14.types.ResponseFlush\x12/\n\x04Info\x12\x12.types.RequestInfo\x1a\x13.types.ResponseInfo\x12>\n\tSetOption\x12\x17.types.RequestSetOption\x1a\x18.types.ResponseSetOption\x12>\n\tDeliverTx\x12\x17.types.RequestDeliverTx\x1a\x18.types.ResponseDeliverTx\x12\x38\n\x07\x43heckTx\x12\x15.types.RequestCheckTx\x1a\x16.types.ResponseCheckTx\x12\x32\n\x05Query\x12\x13.types.RequestQuery\x1a\x14.types.ResponseQuery\x12\x35\n\x06\x43ommit\x12\x14.types.RequestCommit\x1a\x15.types.ResponseCommit\x12>\n\tInitChain\x12\x17.types.RequestInitChain\x1a\x18.types.ResponseInitChain\x12\x41\n\nBeginBlock\x12\x18.types.RequestBeginBlock\x1a\x19.types.ResponseBeginBlock\x12;\n\x08\x45ndBlock\x12\x16.types.RequestEndBlock\x1a\x17.types.ResponseEndBlockb\x06proto3')
  ,
  dependencies=[github_dot_com_dot_gogo_dot_protobuf_dot_gogoproto_dot_gogo__pb2.DESCRIPTOR,github_dot_com_dot_tendermint_dot_tmlibs_dot_common_dot_types__pb2.DESCRIPTOR,])




_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='types.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='echo', full_name='types.Request.echo', index=0,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='flush', full_name='types.Request.flush', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='types.Request.info', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='set_option', full_name='types.Request.set_option', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='init_chain', full_name='types.Request.init_chain', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='query', full_name='types.Request.query', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='begin_block', full_name='types.Request.begin_block', index=6,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='check_tx', full_name='types.Request.check_tx', index=7,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deliver_tx', full_name='types.Request.deliver_tx', index=8,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end_block', full_name='types.Request.end_block', index=9,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='commit', full_name='types.Request.commit', index=10,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='types.Request.value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=119,
  serialized_end=603,
)


_REQUESTECHO = _descriptor.Descriptor(
  name='RequestEcho',
  full_name='types.RequestEcho',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='types.RequestEcho.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=605,
  serialized_end=635,
)


_REQUESTFLUSH = _descriptor.Descriptor(
  name='RequestFlush',
  full_name='types.RequestFlush',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=637,
  serialized_end=651,
)


_REQUESTINFO = _descriptor.Descriptor(
  name='RequestInfo',
  full_name='types.RequestInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='types.RequestInfo.version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=653,
  serialized_end=683,
)


_REQUESTSETOPTION = _descriptor.Descriptor(
  name='RequestSetOption',
  full_name='types.RequestSetOption',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='types.RequestSetOption.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='types.RequestSetOption.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=685,
  serialized_end=731,
)


_REQUESTINITCHAIN = _descriptor.Descriptor(
  name='RequestInitChain',
  full_name='types.RequestInitChain',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='validators', full_name='types.RequestInitChain.validators', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='app_state_bytes', full_name='types.RequestInitChain.app_state_bytes', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=733,
  serialized_end=820,
)


_REQUESTQUERY = _descriptor.Descriptor(
  name='RequestQuery',
  full_name='types.RequestQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='types.RequestQuery.data', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='path', full_name='types.RequestQuery.path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='types.RequestQuery.height', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prove', full_name='types.RequestQuery.prove', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=822,
  serialized_end=895,
)


_REQUESTBEGINBLOCK = _descriptor.Descriptor(
  name='RequestBeginBlock',
  full_name='types.RequestBeginBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='types.RequestBeginBlock.hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header', full_name='types.RequestBeginBlock.header', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='absent_validators', full_name='types.RequestBeginBlock.absent_validators', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='byzantine_validators', full_name='types.RequestBeginBlock.byzantine_validators', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=898,
  serialized_end=1048,
)


_REQUESTCHECKTX = _descriptor.Descriptor(
  name='RequestCheckTx',
  full_name='types.RequestCheckTx',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx', full_name='types.RequestCheckTx.tx', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1050,
  serialized_end=1078,
)


_REQUESTDELIVERTX = _descriptor.Descriptor(
  name='RequestDeliverTx',
  full_name='types.RequestDeliverTx',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx', full_name='types.RequestDeliverTx.tx', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1080,
  serialized_end=1110,
)


_REQUESTENDBLOCK = _descriptor.Descriptor(
  name='RequestEndBlock',
  full_name='types.RequestEndBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='height', full_name='types.RequestEndBlock.height', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1112,
  serialized_end=1145,
)


_REQUESTCOMMIT = _descriptor.Descriptor(
  name='RequestCommit',
  full_name='types.RequestCommit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1147,
  serialized_end=1162,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='types.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='exception', full_name='types.Response.exception', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='echo', full_name='types.Response.echo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='flush', full_name='types.Response.flush', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='types.Response.info', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='set_option', full_name='types.Response.set_option', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='init_chain', full_name='types.Response.init_chain', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='query', full_name='types.Response.query', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='begin_block', full_name='types.Response.begin_block', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='check_tx', full_name='types.Response.check_tx', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deliver_tx', full_name='types.Response.deliver_tx', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end_block', full_name='types.Response.end_block', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='commit', full_name='types.Response.commit', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='types.Response.value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1165,
  serialized_end=1708,
)


_RESPONSEEXCEPTION = _descriptor.Descriptor(
  name='ResponseException',
  full_name='types.ResponseException',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error', full_name='types.ResponseException.error', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1710,
  serialized_end=1744,
)


_RESPONSEECHO = _descriptor.Descriptor(
  name='ResponseEcho',
  full_name='types.ResponseEcho',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='types.ResponseEcho.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1746,
  serialized_end=1777,
)


_RESPONSEFLUSH = _descriptor.Descriptor(
  name='ResponseFlush',
  full_name='types.ResponseFlush',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1779,
  serialized_end=1794,
)


_RESPONSEINFO = _descriptor.Descriptor(
  name='ResponseInfo',
  full_name='types.ResponseInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='types.ResponseInfo.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='types.ResponseInfo.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_block_height', full_name='types.ResponseInfo.last_block_height', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_block_app_hash', full_name='types.ResponseInfo.last_block_app_hash', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1796,
  serialized_end=1897,
)


_RESPONSESETOPTION = _descriptor.Descriptor(
  name='ResponseSetOption',
  full_name='types.ResponseSetOption',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='types.ResponseSetOption.code', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log', full_name='types.ResponseSetOption.log', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='types.ResponseSetOption.info', index=2,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1899,
  serialized_end=1959,
)


_RESPONSEINITCHAIN = _descriptor.Descriptor(
  name='ResponseInitChain',
  full_name='types.ResponseInitChain',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1961,
  serialized_end=1980,
)


_RESPONSEQUERY = _descriptor.Descriptor(
  name='ResponseQuery',
  full_name='types.ResponseQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='types.ResponseQuery.code', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log', full_name='types.ResponseQuery.log', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='types.ResponseQuery.info', index=2,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='index', full_name='types.ResponseQuery.index', index=3,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key', full_name='types.ResponseQuery.key', index=4,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='types.ResponseQuery.value', index=5,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proof', full_name='types.ResponseQuery.proof', index=6,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='types.ResponseQuery.height', index=7,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1983,
  serialized_end=2113,
)


_RESPONSEBEGINBLOCK = _descriptor.Descriptor(
  name='ResponseBeginBlock',
  full_name='types.ResponseBeginBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2115,
  serialized_end=2135,
)


_RESPONSECHECKTX = _descriptor.Descriptor(
  name='ResponseCheckTx',
  full_name='types.ResponseCheckTx',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='types.ResponseCheckTx.code', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='types.ResponseCheckTx.data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log', full_name='types.ResponseCheckTx.log', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='types.ResponseCheckTx.info', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gas_wanted', full_name='types.ResponseCheckTx.gas_wanted', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gas_used', full_name='types.ResponseCheckTx.gas_used', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='types.ResponseCheckTx.tags', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000\352\336\037\016tags,omitempty')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fee', full_name='types.ResponseCheckTx.fee', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2138,
  serialized_end=2339,
)


_RESPONSEDELIVERTX = _descriptor.Descriptor(
  name='ResponseDeliverTx',
  full_name='types.ResponseDeliverTx',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='types.ResponseDeliverTx.code', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='types.ResponseDeliverTx.data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log', full_name='types.ResponseDeliverTx.log', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='types.ResponseDeliverTx.info', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gas_wanted', full_name='types.ResponseDeliverTx.gas_wanted', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gas_used', full_name='types.ResponseDeliverTx.gas_used', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='types.ResponseDeliverTx.tags', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000\352\336\037\016tags,omitempty')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fee', full_name='types.ResponseDeliverTx.fee', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2342,
  serialized_end=2545,
)


_RESPONSEENDBLOCK = _descriptor.Descriptor(
  name='ResponseEndBlock',
  full_name='types.ResponseEndBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='validator_updates', full_name='types.ResponseEndBlock.validator_updates', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='consensus_param_updates', full_name='types.ResponseEndBlock.consensus_param_updates', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2547,
  serialized_end=2673,
)


_RESPONSECOMMIT = _descriptor.Descriptor(
  name='ResponseCommit',
  full_name='types.ResponseCommit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='types.ResponseCommit.data', index=0,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2675,
  serialized_end=2705,
)


_CONSENSUSPARAMS = _descriptor.Descriptor(
  name='ConsensusParams',
  full_name='types.ConsensusParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_size', full_name='types.ConsensusParams.block_size', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tx_size', full_name='types.ConsensusParams.tx_size', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='block_gossip', full_name='types.ConsensusParams.block_gossip', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2708,
  serialized_end=2837,
)


_BLOCKSIZE = _descriptor.Descriptor(
  name='BlockSize',
  full_name='types.BlockSize',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='max_bytes', full_name='types.BlockSize.max_bytes', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_txs', full_name='types.BlockSize.max_txs', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_gas', full_name='types.BlockSize.max_gas', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2839,
  serialized_end=2903,
)


_TXSIZE = _descriptor.Descriptor(
  name='TxSize',
  full_name='types.TxSize',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='max_bytes', full_name='types.TxSize.max_bytes', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_gas', full_name='types.TxSize.max_gas', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2905,
  serialized_end=2949,
)


_BLOCKGOSSIP = _descriptor.Descriptor(
  name='BlockGossip',
  full_name='types.BlockGossip',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_part_size_bytes', full_name='types.BlockGossip.block_part_size_bytes', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2951,
  serialized_end=2995,
)


_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='types.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chain_id', full_name='types.Header.chain_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\336\037\007ChainID')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='types.Header.height', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='types.Header.time', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_txs', full_name='types.Header.num_txs', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_block_id', full_name='types.Header.last_block_id', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\336\037\013LastBlockID\310\336\037\000')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_commit_hash', full_name='types.Header.last_commit_hash', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data_hash', full_name='types.Header.data_hash', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='validators_hash', full_name='types.Header.validators_hash', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='app_hash', full_name='types.Header.app_hash', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2998,
  serialized_end=3232,
)


_BLOCKID = _descriptor.Descriptor(
  name='BlockID',
  full_name='types.BlockID',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='types.BlockID.hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parts', full_name='types.BlockID.parts', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3234,
  serialized_end=3300,
)


_PARTSETHEADER = _descriptor.Descriptor(
  name='PartSetHeader',
  full_name='types.PartSetHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='total', full_name='types.PartSetHeader.total', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hash', full_name='types.PartSetHeader.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3302,
  serialized_end=3346,
)


_VALIDATOR = _descriptor.Descriptor(
  name='Validator',
  full_name='types.Validator',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pub_key', full_name='types.Validator.pub_key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='power', full_name='types.Validator.power', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3348,
  serialized_end=3391,
)


_EVIDENCE = _descriptor.Descriptor(
  name='Evidence',
  full_name='types.Evidence',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pub_key', full_name='types.Evidence.pub_key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='types.Evidence.height', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3393,
  serialized_end=3436,
)

_REQUEST.fields_by_name['echo'].message_type = _REQUESTECHO
_REQUEST.fields_by_name['flush'].message_type = _REQUESTFLUSH
_REQUEST.fields_by_name['info'].message_type = _REQUESTINFO
_REQUEST.fields_by_name['set_option'].message_type = _REQUESTSETOPTION
_REQUEST.fields_by_name['init_chain'].message_type = _REQUESTINITCHAIN
_REQUEST.fields_by_name['query'].message_type = _REQUESTQUERY
_REQUEST.fields_by_name['begin_block'].message_type = _REQUESTBEGINBLOCK
_REQUEST.fields_by_name['check_tx'].message_type = _REQUESTCHECKTX
_REQUEST.fields_by_name['deliver_tx'].message_type = _REQUESTDELIVERTX
_REQUEST.fields_by_name['end_block'].message_type = _REQUESTENDBLOCK
_REQUEST.fields_by_name['commit'].message_type = _REQUESTCOMMIT
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['echo'])
_REQUEST.fields_by_name['echo'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['flush'])
_REQUEST.fields_by_name['flush'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['info'])
_REQUEST.fields_by_name['info'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['set_option'])
_REQUEST.fields_by_name['set_option'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['init_chain'])
_REQUEST.fields_by_name['init_chain'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['query'])
_REQUEST.fields_by_name['query'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['begin_block'])
_REQUEST.fields_by_name['begin_block'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['check_tx'])
_REQUEST.fields_by_name['check_tx'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['deliver_tx'])
_REQUEST.fields_by_name['deliver_tx'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['end_block'])
_REQUEST.fields_by_name['end_block'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUEST.oneofs_by_name['value'].fields.append(
  _REQUEST.fields_by_name['commit'])
_REQUEST.fields_by_name['commit'].containing_oneof = _REQUEST.oneofs_by_name['value']
_REQUESTINITCHAIN.fields_by_name['validators'].message_type = _VALIDATOR
_REQUESTBEGINBLOCK.fields_by_name['header'].message_type = _HEADER
_REQUESTBEGINBLOCK.fields_by_name['byzantine_validators'].message_type = _EVIDENCE
_RESPONSE.fields_by_name['exception'].message_type = _RESPONSEEXCEPTION
_RESPONSE.fields_by_name['echo'].message_type = _RESPONSEECHO
_RESPONSE.fields_by_name['flush'].message_type = _RESPONSEFLUSH
_RESPONSE.fields_by_name['info'].message_type = _RESPONSEINFO
_RESPONSE.fields_by_name['set_option'].message_type = _RESPONSESETOPTION
_RESPONSE.fields_by_name['init_chain'].message_type = _RESPONSEINITCHAIN
_RESPONSE.fields_by_name['query'].message_type = _RESPONSEQUERY
_RESPONSE.fields_by_name['begin_block'].message_type = _RESPONSEBEGINBLOCK
_RESPONSE.fields_by_name['check_tx'].message_type = _RESPONSECHECKTX
_RESPONSE.fields_by_name['deliver_tx'].message_type = _RESPONSEDELIVERTX
_RESPONSE.fields_by_name['end_block'].message_type = _RESPONSEENDBLOCK
_RESPONSE.fields_by_name['commit'].message_type = _RESPONSECOMMIT
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['exception'])
_RESPONSE.fields_by_name['exception'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['echo'])
_RESPONSE.fields_by_name['echo'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['flush'])
_RESPONSE.fields_by_name['flush'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['info'])
_RESPONSE.fields_by_name['info'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['set_option'])
_RESPONSE.fields_by_name['set_option'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['init_chain'])
_RESPONSE.fields_by_name['init_chain'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['query'])
_RESPONSE.fields_by_name['query'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['begin_block'])
_RESPONSE.fields_by_name['begin_block'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['check_tx'])
_RESPONSE.fields_by_name['check_tx'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['deliver_tx'])
_RESPONSE.fields_by_name['deliver_tx'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['end_block'])
_RESPONSE.fields_by_name['end_block'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSE.oneofs_by_name['value'].fields.append(
  _RESPONSE.fields_by_name['commit'])
_RESPONSE.fields_by_name['commit'].containing_oneof = _RESPONSE.oneofs_by_name['value']
_RESPONSECHECKTX.fields_by_name['tags'].message_type = github_dot_com_dot_tendermint_dot_tmlibs_dot_common_dot_types__pb2._KVPAIR
_RESPONSECHECKTX.fields_by_name['fee'].message_type = github_dot_com_dot_tendermint_dot_tmlibs_dot_common_dot_types__pb2._KI64PAIR
_RESPONSEDELIVERTX.fields_by_name['tags'].message_type = github_dot_com_dot_tendermint_dot_tmlibs_dot_common_dot_types__pb2._KVPAIR
_RESPONSEDELIVERTX.fields_by_name['fee'].message_type = github_dot_com_dot_tendermint_dot_tmlibs_dot_common_dot_types__pb2._KI64PAIR
_RESPONSEENDBLOCK.fields_by_name['validator_updates'].message_type = _VALIDATOR
_RESPONSEENDBLOCK.fields_by_name['consensus_param_updates'].message_type = _CONSENSUSPARAMS
_CONSENSUSPARAMS.fields_by_name['block_size'].message_type = _BLOCKSIZE
_CONSENSUSPARAMS.fields_by_name['tx_size'].message_type = _TXSIZE
_CONSENSUSPARAMS.fields_by_name['block_gossip'].message_type = _BLOCKGOSSIP
_HEADER.fields_by_name['last_block_id'].message_type = _BLOCKID
_BLOCKID.fields_by_name['parts'].message_type = _PARTSETHEADER
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['RequestEcho'] = _REQUESTECHO
DESCRIPTOR.message_types_by_name['RequestFlush'] = _REQUESTFLUSH
DESCRIPTOR.message_types_by_name['RequestInfo'] = _REQUESTINFO
DESCRIPTOR.message_types_by_name['RequestSetOption'] = _REQUESTSETOPTION
DESCRIPTOR.message_types_by_name['RequestInitChain'] = _REQUESTINITCHAIN
DESCRIPTOR.message_types_by_name['RequestQuery'] = _REQUESTQUERY
DESCRIPTOR.message_types_by_name['RequestBeginBlock'] = _REQUESTBEGINBLOCK
DESCRIPTOR.message_types_by_name['RequestCheckTx'] = _REQUESTCHECKTX
DESCRIPTOR.message_types_by_name['RequestDeliverTx'] = _REQUESTDELIVERTX
DESCRIPTOR.message_types_by_name['RequestEndBlock'] = _REQUESTENDBLOCK
DESCRIPTOR.message_types_by_name['RequestCommit'] = _REQUESTCOMMIT
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.message_types_by_name['ResponseException'] = _RESPONSEEXCEPTION
DESCRIPTOR.message_types_by_name['ResponseEcho'] = _RESPONSEECHO
DESCRIPTOR.message_types_by_name['ResponseFlush'] = _RESPONSEFLUSH
DESCRIPTOR.message_types_by_name['ResponseInfo'] = _RESPONSEINFO
DESCRIPTOR.message_types_by_name['ResponseSetOption'] = _RESPONSESETOPTION
DESCRIPTOR.message_types_by_name['ResponseInitChain'] = _RESPONSEINITCHAIN
DESCRIPTOR.message_types_by_name['ResponseQuery'] = _RESPONSEQUERY
DESCRIPTOR.message_types_by_name['ResponseBeginBlock'] = _RESPONSEBEGINBLOCK
DESCRIPTOR.message_types_by_name['ResponseCheckTx'] = _RESPONSECHECKTX
DESCRIPTOR.message_types_by_name['ResponseDeliverTx'] = _RESPONSEDELIVERTX
DESCRIPTOR.message_types_by_name['ResponseEndBlock'] = _RESPONSEENDBLOCK
DESCRIPTOR.message_types_by_name['ResponseCommit'] = _RESPONSECOMMIT
DESCRIPTOR.message_types_by_name['ConsensusParams'] = _CONSENSUSPARAMS
DESCRIPTOR.message_types_by_name['BlockSize'] = _BLOCKSIZE
DESCRIPTOR.message_types_by_name['TxSize'] = _TXSIZE
DESCRIPTOR.message_types_by_name['BlockGossip'] = _BLOCKGOSSIP
DESCRIPTOR.message_types_by_name['Header'] = _HEADER
DESCRIPTOR.message_types_by_name['BlockID'] = _BLOCKID
DESCRIPTOR.message_types_by_name['PartSetHeader'] = _PARTSETHEADER
DESCRIPTOR.message_types_by_name['Validator'] = _VALIDATOR
DESCRIPTOR.message_types_by_name['Evidence'] = _EVIDENCE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.Request)
  ))
_sym_db.RegisterMessage(Request)

RequestEcho = _reflection.GeneratedProtocolMessageType('RequestEcho', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTECHO,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestEcho)
  ))
_sym_db.RegisterMessage(RequestEcho)

RequestFlush = _reflection.GeneratedProtocolMessageType('RequestFlush', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTFLUSH,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestFlush)
  ))
_sym_db.RegisterMessage(RequestFlush)

RequestInfo = _reflection.GeneratedProtocolMessageType('RequestInfo', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTINFO,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestInfo)
  ))
_sym_db.RegisterMessage(RequestInfo)

RequestSetOption = _reflection.GeneratedProtocolMessageType('RequestSetOption', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTSETOPTION,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestSetOption)
  ))
_sym_db.RegisterMessage(RequestSetOption)

RequestInitChain = _reflection.GeneratedProtocolMessageType('RequestInitChain', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTINITCHAIN,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestInitChain)
  ))
_sym_db.RegisterMessage(RequestInitChain)

RequestQuery = _reflection.GeneratedProtocolMessageType('RequestQuery', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTQUERY,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestQuery)
  ))
_sym_db.RegisterMessage(RequestQuery)

RequestBeginBlock = _reflection.GeneratedProtocolMessageType('RequestBeginBlock', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTBEGINBLOCK,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestBeginBlock)
  ))
_sym_db.RegisterMessage(RequestBeginBlock)

RequestCheckTx = _reflection.GeneratedProtocolMessageType('RequestCheckTx', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTCHECKTX,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestCheckTx)
  ))
_sym_db.RegisterMessage(RequestCheckTx)

RequestDeliverTx = _reflection.GeneratedProtocolMessageType('RequestDeliverTx', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTDELIVERTX,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestDeliverTx)
  ))
_sym_db.RegisterMessage(RequestDeliverTx)

RequestEndBlock = _reflection.GeneratedProtocolMessageType('RequestEndBlock', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTENDBLOCK,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestEndBlock)
  ))
_sym_db.RegisterMessage(RequestEndBlock)

RequestCommit = _reflection.GeneratedProtocolMessageType('RequestCommit', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTCOMMIT,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.RequestCommit)
  ))
_sym_db.RegisterMessage(RequestCommit)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.Response)
  ))
_sym_db.RegisterMessage(Response)

ResponseException = _reflection.GeneratedProtocolMessageType('ResponseException', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEEXCEPTION,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseException)
  ))
_sym_db.RegisterMessage(ResponseException)

ResponseEcho = _reflection.GeneratedProtocolMessageType('ResponseEcho', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEECHO,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseEcho)
  ))
_sym_db.RegisterMessage(ResponseEcho)

ResponseFlush = _reflection.GeneratedProtocolMessageType('ResponseFlush', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEFLUSH,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseFlush)
  ))
_sym_db.RegisterMessage(ResponseFlush)

ResponseInfo = _reflection.GeneratedProtocolMessageType('ResponseInfo', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEINFO,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseInfo)
  ))
_sym_db.RegisterMessage(ResponseInfo)

ResponseSetOption = _reflection.GeneratedProtocolMessageType('ResponseSetOption', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSESETOPTION,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseSetOption)
  ))
_sym_db.RegisterMessage(ResponseSetOption)

ResponseInitChain = _reflection.GeneratedProtocolMessageType('ResponseInitChain', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEINITCHAIN,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseInitChain)
  ))
_sym_db.RegisterMessage(ResponseInitChain)

ResponseQuery = _reflection.GeneratedProtocolMessageType('ResponseQuery', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEQUERY,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseQuery)
  ))
_sym_db.RegisterMessage(ResponseQuery)

ResponseBeginBlock = _reflection.GeneratedProtocolMessageType('ResponseBeginBlock', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEBEGINBLOCK,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseBeginBlock)
  ))
_sym_db.RegisterMessage(ResponseBeginBlock)

ResponseCheckTx = _reflection.GeneratedProtocolMessageType('ResponseCheckTx', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSECHECKTX,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseCheckTx)
  ))
_sym_db.RegisterMessage(ResponseCheckTx)

ResponseDeliverTx = _reflection.GeneratedProtocolMessageType('ResponseDeliverTx', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEDELIVERTX,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseDeliverTx)
  ))
_sym_db.RegisterMessage(ResponseDeliverTx)

ResponseEndBlock = _reflection.GeneratedProtocolMessageType('ResponseEndBlock', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEENDBLOCK,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseEndBlock)
  ))
_sym_db.RegisterMessage(ResponseEndBlock)

ResponseCommit = _reflection.GeneratedProtocolMessageType('ResponseCommit', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSECOMMIT,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ResponseCommit)
  ))
_sym_db.RegisterMessage(ResponseCommit)

ConsensusParams = _reflection.GeneratedProtocolMessageType('ConsensusParams', (_message.Message,), dict(
  DESCRIPTOR = _CONSENSUSPARAMS,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.ConsensusParams)
  ))
_sym_db.RegisterMessage(ConsensusParams)

BlockSize = _reflection.GeneratedProtocolMessageType('BlockSize', (_message.Message,), dict(
  DESCRIPTOR = _BLOCKSIZE,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.BlockSize)
  ))
_sym_db.RegisterMessage(BlockSize)

TxSize = _reflection.GeneratedProtocolMessageType('TxSize', (_message.Message,), dict(
  DESCRIPTOR = _TXSIZE,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.TxSize)
  ))
_sym_db.RegisterMessage(TxSize)

BlockGossip = _reflection.GeneratedProtocolMessageType('BlockGossip', (_message.Message,), dict(
  DESCRIPTOR = _BLOCKGOSSIP,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.BlockGossip)
  ))
_sym_db.RegisterMessage(BlockGossip)

Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), dict(
  DESCRIPTOR = _HEADER,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.Header)
  ))
_sym_db.RegisterMessage(Header)

BlockID = _reflection.GeneratedProtocolMessageType('BlockID', (_message.Message,), dict(
  DESCRIPTOR = _BLOCKID,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.BlockID)
  ))
_sym_db.RegisterMessage(BlockID)

PartSetHeader = _reflection.GeneratedProtocolMessageType('PartSetHeader', (_message.Message,), dict(
  DESCRIPTOR = _PARTSETHEADER,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.PartSetHeader)
  ))
_sym_db.RegisterMessage(PartSetHeader)

Validator = _reflection.GeneratedProtocolMessageType('Validator', (_message.Message,), dict(
  DESCRIPTOR = _VALIDATOR,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.Validator)
  ))
_sym_db.RegisterMessage(Validator)

Evidence = _reflection.GeneratedProtocolMessageType('Evidence', (_message.Message,), dict(
  DESCRIPTOR = _EVIDENCE,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:types.Evidence)
  ))
_sym_db.RegisterMessage(Evidence)


_REQUESTINITCHAIN.fields_by_name['validators'].has_options = True
_REQUESTINITCHAIN.fields_by_name['validators']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000'))
_REQUESTBEGINBLOCK.fields_by_name['header'].has_options = True
_REQUESTBEGINBLOCK.fields_by_name['header']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000'))
_REQUESTBEGINBLOCK.fields_by_name['byzantine_validators'].has_options = True
_REQUESTBEGINBLOCK.fields_by_name['byzantine_validators']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000'))
_RESPONSECHECKTX.fields_by_name['tags'].has_options = True
_RESPONSECHECKTX.fields_by_name['tags']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000\352\336\037\016tags,omitempty'))
_RESPONSECHECKTX.fields_by_name['fee'].has_options = True
_RESPONSECHECKTX.fields_by_name['fee']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000'))
_RESPONSEDELIVERTX.fields_by_name['tags'].has_options = True
_RESPONSEDELIVERTX.fields_by_name['tags']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000\352\336\037\016tags,omitempty'))
_RESPONSEDELIVERTX.fields_by_name['fee'].has_options = True
_RESPONSEDELIVERTX.fields_by_name['fee']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000'))
_RESPONSEENDBLOCK.fields_by_name['validator_updates'].has_options = True
_RESPONSEENDBLOCK.fields_by_name['validator_updates']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000'))
_HEADER.fields_by_name['chain_id'].has_options = True
_HEADER.fields_by_name['chain_id']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\336\037\007ChainID'))
_HEADER.fields_by_name['last_block_id'].has_options = True
_HEADER.fields_by_name['last_block_id']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\336\037\013LastBlockID\310\336\037\000'))
_BLOCKID.fields_by_name['parts'].has_options = True
_BLOCKID.fields_by_name['parts']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\310\336\037\000'))

_ABCIAPPLICATION = _descriptor.ServiceDescriptor(
  name='ABCIApplication',
  full_name='types.ABCIApplication',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=3439,
  serialized_end=4091,
  methods=[
  _descriptor.MethodDescriptor(
    name='Echo',
    full_name='types.ABCIApplication.Echo',
    index=0,
    containing_service=None,
    input_type=_REQUESTECHO,
    output_type=_RESPONSEECHO,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Flush',
    full_name='types.ABCIApplication.Flush',
    index=1,
    containing_service=None,
    input_type=_REQUESTFLUSH,
    output_type=_RESPONSEFLUSH,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Info',
    full_name='types.ABCIApplication.Info',
    index=2,
    containing_service=None,
    input_type=_REQUESTINFO,
    output_type=_RESPONSEINFO,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SetOption',
    full_name='types.ABCIApplication.SetOption',
    index=3,
    containing_service=None,
    input_type=_REQUESTSETOPTION,
    output_type=_RESPONSESETOPTION,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeliverTx',
    full_name='types.ABCIApplication.DeliverTx',
    index=4,
    containing_service=None,
    input_type=_REQUESTDELIVERTX,
    output_type=_RESPONSEDELIVERTX,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CheckTx',
    full_name='types.ABCIApplication.CheckTx',
    index=5,
    containing_service=None,
    input_type=_REQUESTCHECKTX,
    output_type=_RESPONSECHECKTX,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Query',
    full_name='types.ABCIApplication.Query',
    index=6,
    containing_service=None,
    input_type=_REQUESTQUERY,
    output_type=_RESPONSEQUERY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Commit',
    full_name='types.ABCIApplication.Commit',
    index=7,
    containing_service=None,
    input_type=_REQUESTCOMMIT,
    output_type=_RESPONSECOMMIT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='InitChain',
    full_name='types.ABCIApplication.InitChain',
    index=8,
    containing_service=None,
    input_type=_REQUESTINITCHAIN,
    output_type=_RESPONSEINITCHAIN,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='BeginBlock',
    full_name='types.ABCIApplication.BeginBlock',
    index=9,
    containing_service=None,
    input_type=_REQUESTBEGINBLOCK,
    output_type=_RESPONSEBEGINBLOCK,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='EndBlock',
    full_name='types.ABCIApplication.EndBlock',
    index=10,
    containing_service=None,
    input_type=_REQUESTENDBLOCK,
    output_type=_RESPONSEENDBLOCK,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_ABCIAPPLICATION)

DESCRIPTOR.services_by_name['ABCIApplication'] = _ABCIAPPLICATION

# @@protoc_insertion_point(module_scope)




import struct
from math import ceil
import binascii, logging, colorlog

def str_to_bytes(data):
    if isinstance(data, str):
        return data.encode('utf-8')
    return data

def bytes_to_str(value):
    if isinstance(value, str):
        return value
    return value.decode('utf-8')

def int_to_big_endian(value):
    byte_length = max(ceil(value.bit_length() / 8), 1)
    return (value).to_bytes(byte_length, byteorder='big')

def big_endian_to_int(value):
    return int.from_bytes(value, byteorder='big')

def decode_hex(s):
    if isinstance(s, str):
        return bytes.fromhex(s)
    if isinstance(s, (bytes, bytearray)):
        return binascii.unhexlify(s)
    raise TypeError('Value must be an instance of str or bytes')

def encode_hex(b):
    if isinstance(b, str):
        b = bytes(b, 'utf-8')
    if isinstance(b, (bytes, bytearray)):
        return str(binascii.hexlify(b), 'utf-8')
    raise TypeError('Value must be an instance of str or bytes')

def get_logger():
    logger = logging.getLogger('abci.app')

    if logger.hasHandlers():
        return logger

    formatter = colorlog.ColoredFormatter(
        "%(log_color)s%(levelname)-8s%(reset)s %(white)s%(message)s",
        datefmt=None,
        reset=True,
        log_colors={
            'DEBUG':    'cyan',
		    'INFO':     'green',
		    'WARNING':  'yellow',
		    'ERROR':    'red',
		    'CRITICAL': 'red,bg_white',
        },
        secondary_log_colors={},
        style='%')

    handler = logging.StreamHandler()
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)

    return logger



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: github.com/tendermint/tmlibs/common/types.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from github.com.gogo.protobuf.gogoproto import gogo_pb2 as github_dot_com_dot_gogo_dot_protobuf_dot_gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='github.com/tendermint/tmlibs/common/types.proto',
  package='common',
  syntax='proto3',
  serialized_pb=_b('\n/github.com/tendermint/tmlibs/common/types.proto\x12\x06\x63ommon\x1a-github.com/gogo/protobuf/gogoproto/gogo.proto\"$\n\x06KVPair\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\"&\n\x08KI64Pair\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x03\x62\x06proto3')
  ,
  dependencies=[github_dot_com_dot_gogo_dot_protobuf_dot_gogoproto_dot_gogo__pb2.DESCRIPTOR,])




_KVPAIR = _descriptor.Descriptor(
  name='KVPair',
  full_name='common.KVPair',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='common.KVPair.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.KVPair.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=106,
  serialized_end=142,
)


_KI64PAIR = _descriptor.Descriptor(
  name='KI64Pair',
  full_name='common.KI64Pair',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='common.KI64Pair.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.KI64Pair.value', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=144,
  serialized_end=182,
)

DESCRIPTOR.message_types_by_name['KVPair'] = _KVPAIR
DESCRIPTOR.message_types_by_name['KI64Pair'] = _KI64PAIR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

KVPair = _reflection.GeneratedProtocolMessageType('KVPair', (_message.Message,), dict(
  DESCRIPTOR = _KVPAIR,
  __module__ = 'github.com.tendermint.tmlibs.common.types_pb2'
  # @@protoc_insertion_point(class_scope:common.KVPair)
  ))
_sym_db.RegisterMessage(KVPair)

KI64Pair = _reflection.GeneratedProtocolMessageType('KI64Pair', (_message.Message,), dict(
  DESCRIPTOR = _KI64PAIR,
  __module__ = 'github.com.tendermint.tmlibs.common.types_pb2'
  # @@protoc_insertion_point(class_scope:common.KI64Pair)
  ))
_sym_db.RegisterMessage(KI64Pair)


# @@protoc_insertion_point(module_scope)



"""
ABCI TCP server
Tendermint connects to this server over 3 different connections:
 - mempool: used for check_tx
 - consensus: used for the begin_block -> deliver_tx -> end_block -> commit flow
 - query: used to query application state

This can be a bit confusing in the app since gevent spawns a greenlet for each
connection.  If one crashes you will not have full connectivity to Tendermint
"""
import sys
import struct
import signal
from io import BytesIO

import gevent
from gevent.event import Event
from gevent.server import StreamServer

from .encoding import (
    read_message,
    write_message,
    NODATA,
    FRAGDATA,
    OK,
)

from .utils import get_logger
from .application import BaseApplication

from .types_pb2 import (
    Request, Response, ResponseException,
    RequestEcho, ResponseEcho,
    RequestFlush, ResponseFlush,
    RequestInitChain, ResponseInitChain,
    RequestInfo, ResponseInfo,
    RequestSetOption, ResponseSetOption,
    ResponseDeliverTx,
    ResponseCheckTx,
    RequestQuery, ResponseQuery,
    RequestBeginBlock, ResponseBeginBlock,
    RequestEndBlock, ResponseEndBlock,
    ResponseCommit,
)

log = get_logger()

class ProtocolHandler:
    """ Internal handler called by the server to process requests from
    Tendermint.  The handler delegates call to your application"""
    def __init__(self, app):
        self.app = app

    def process(self,req_type, req):
        handler = getattr(self, req_type, self.no_match)
        return handler(req)

    def echo(self, req):
        msg = req.echo.message
        response = Response(echo=ResponseEcho(message=msg))
        return write_message(response)

    def flush(self, req):
        response = Response(flush=ResponseFlush())
        return write_message(response)

    def info(self, req):
        result = self.app.info(req.info)
        response = Response(info=result)
        return write_message(response)

    def set_option(self, req):
        result = self.app.set_option(req.set_option)
        response = Response(set_option=result)
        return write_message(response)

    def check_tx(self, req):
        result = self.app.check_tx(req.check_tx.tx)
        response = Response(check_tx=result)
        return write_message(response)

    def deliver_tx(self, req):
        result = self.app.deliver_tx(req.deliver_tx.tx)
        response = Response(deliver_tx=result)
        return write_message(response)

    def query(self, req):
        result = self.app.query(req.query)
        response = Response(query=result)
        return write_message(response)

    def commit(self, req):
        result = self.app.commit()
        response = Response(commit=result)
        return write_message(response)

    def begin_block(self, req):
        result = self.app.begin_block(req.begin_block)
        response = Response(begin_block=result)
        return write_message(response)

    def end_block(self, req):
        result = self.app.end_block(req.end_block)
        response = Response(end_block=result)
        return write_message(response)

    def init_chain(self, req):
        result = self.app.init_chain(req.init_chain)
        response = Response(init_chain=result)
        return write_message(response)

    def no_match(self, req):
        response = Response(exception=ResponseException(error="ABCI request not found"))
        return write_message(response)


class ABCIServer:
    def __init__(self, port=46658, app=None):
        if not app or not isinstance(app, BaseApplication):
            log.error("Application missing or not an instance of Base Application")
            raise TypeError("Application missing or not an instance of Base Application")
        self.port = port
        self.protocol = ProtocolHandler(app)
        self.server = StreamServer(('0.0.0.0', port), handle=self.__handle_connection)

    def start(self):
        self.server.start()

    def stop(self):
        log.info("Shutting down server")
        self.server.stop()

    def run(self):
        """Option to calling manually calling start()/stop(). This will start
        the server and watch for signals to stop the server"""
        self.server.start()
        log.info(" ABCIServer started on port: {}".format(self.port))

        # wait for interrupt
        evt = Event()
        gevent.signal(signal.SIGQUIT, evt.set)
        gevent.signal(signal.SIGTERM, evt.set)
        gevent.signal(signal.SIGINT, evt.set)
        evt.wait()

        log.info("Shutting down server")
        self.server.stop()

    # TM will spawn off 3 connections: mempool, consensus, query
    # If an error happens in 1 it still leaves the others open which
    # means you don't have all the connections available to TM
    def __handle_connection(self, socket, address):
        log.info(' ... connection from Tendermint: {}:{} ...'.format(address[0], address[1]))
        data = BytesIO()
        carry_forward = b''
        while True:
            inbound = socket.recv(1024)
            msg_length = len(carry_forward) + len(inbound)
            data.write(carry_forward)
            data.write(inbound)

            data.seek(0)
            carry_forward = b''
            if not data or msg_length == 0: return
            try:
                while data.tell() < msg_length:
                    result, code  = read_message(data, Request)
                    if code == NODATA: return
                    if code == FRAGDATA:
                        data.seek(result)
                        carry_forward = data.read(1024)
                        break
                    req_type = result.WhichOneof("value")
                    response = self.protocol.process(req_type, result)
                    socket.send(response)
            except:
                log.error(" Server Error: {}".format(sys.exc_info()[1]))

            data.seek(0)
            data.truncate()
        socket.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from codecs import open
from os import path

DIR = path.abspath(path.dirname(__file__))

with open(path.join(DIR, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='abci',
    version='0.5.0',
    description='Python based ABCI Server for Tendermint',
    long_description=long_description,
    url='https://github.com/davebryson/py-abci',
    author='Dave Bryson',
    license='Apache 2.0',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='blockchain tendermint abci',
    packages=find_packages(exclude=['tests']),
    install_requires=[
        "protobuf>=3.5.2.post1",
        "gevent>=1.3a2",
        "colorlog>=3.1.2",
        "pytest>=3.5.0",
        "pytest-pythonpath>=0.7.2",
        "pytest-cov>=2.5.1"
    ],
    python_requires='>=3.6',
)



from io import BytesIO

# Codes
NODATA = 1
FRAGDATA = 2
OK = 3

def encode_varint(number):
    # Shift to int64
    number = number << 1
    buf = b''
    while True:
        towrite = number & 0x7f
        number >>= 7
        if number:
            buf += bytes((towrite | 0x80,))
        else:
            buf += bytes((towrite,))
            break
    return buf

def decode_varint(stream):
    shift = 0
    result = 0
    while True:
        i = _read_one(stream)
        result |= (i & 0x7f) << shift
        shift += 7
        if not (i & 0x80):
            break
    return result

def _read_one(stream):
    c = stream.read(1)
    if c == b'':
        raise EOFError("Unexpected EOF while reading bytes")
    return ord(c)

def write_message(message):
    buffer = BytesIO(b'')
    bz = message.SerializeToString()
    buffer.write(encode_varint(len(bz)))
    buffer.write(bz)
    return buffer.getvalue()

def read_message(reader, message):
    """ read the message based on the varintself.
    Returns: (value, code) based on results
    """
    current_position = reader.tell()
    len64 = decode_varint(reader)
    length = len64 >> 1
    slice = reader.read(length)

    if len(slice) == 0:
        return None, NODATA

    if len(slice) < length:
        return current_position, FRAGDATA

    m = message()
    m.ParseFromString(slice)
    return m, OK
